@@ -1,7 +1,9 @@
 import concurrent.futures as pyfutures
+import copy
 import json
 import math
 import os
+from concurrent.futures import as_completed
 from datetime import datetime
 from typing import Dict, List
 
@@ -19,7 +21,7 @@ class RefuelClient:
     API_KEY_ENV_VARIABLE = "REFUEL_API_KEY"
     TIMEOUT_SECS = 15
     MAX_RETRIES = 3
-    MAX_WORKERS = 4 * os.cpu_count()
+    MAX_WORKERS = 4
     DT_FORMAT = "%Y-%m-%dT%H:%M:%S.%fZ"
     MAX_BYTES_PER_BATCH = 3000000  # 3 MB
     MAX_EVENTS_PER_BATCH = 100
@@ -73,6 +75,9 @@ class RefuelClient:
             url, headers=self._header, timeout=self._timeout, data=body
         )
 
+    def _delete(self, url: str, params: Dict = None) -> pyfutures.Future:
+        return self._session.delete(url, headers=self._header, params=params)
+
     def _chunkify_events(self, events: List) -> List:
         if len(events) == 0:
             return []
@@ -90,7 +95,7 @@ class RefuelClient:
         self,
         model_name: str,
         x: Dict,
-        y_pred: Dict,
+        y_pred: Dict = None,
         y_true: Dict = None,
         metadata: Dict = None,
         dataset_name: str = DEFAULT_DATASET_NAME,
@@ -102,7 +107,7 @@ class RefuelClient:
             model_name (str): _description_
             dataset_name (str): _description_
             x (dict): _description_
-            y_pred (dict): _description_
+            y_pred (dict): _description_. Defaults to None.
             y_true (dict, optional): _description_. Defaults to None.
             metadata (dict, optional): _description_. Defaults to None.
 
@@ -159,14 +164,14 @@ class RefuelClient:
         return futures
 
     def log_dataset(
-        self, model_name: str, dataset_name: str, s3_path: str, dataset_config: dict
+        self, model_name: str, dataset_name: str, cloud_uri: str, dataset_config: dict
     ) -> pyfutures.Future:
         """Log dataset to the Refuel platform.
 
         Args:
             model_name (str): model to log dataset to
             dataset_name (str): name of dataset
-            s3_path (str): s3 path to dataset
+            cloud_uri (str): cloud uri or presigned url to dataset
             dataset_config (dict): configuration for the dataset
 
         Returns:
@@ -184,18 +189,15 @@ class RefuelClient:
         client_timestamp = datetime.utcnow().strftime(self.DT_FORMAT)
 
         dataset_event = DatasetEvent(
-            model_name, dataset_name, client_timestamp, s3_path, dataset_config
+            model_name, dataset_name, client_timestamp, cloud_uri, dataset_config
         )
 
         return self._post(
             url=self._api_base_url + "/log_dataset", body=dataset_event.serialize()
         )
 
-    def _query_helper(self, url: str, params: Dict = None) -> pd.DataFrame:
+    def _event_count_helper(self, url: str, params: Dict = None) -> int:
         model_name = params.get("model_name", None)
-        max_events = params.get("max_events")
-        offset = params.get("offset", 0)
-
         if not self._api_key:
             logger.error(
                 f"API Key is absent or invalid: {self._api_key}. No event was logged"
@@ -205,36 +207,78 @@ class RefuelClient:
             logger.error(f"Model name: {model_name} does not exist for this team")
             return None
 
-        events = []
-        while len(events) < max_events:
-            params["offset"] = len(events) + offset
-            params["max_events"] = (
-                min(max_events - len(events), RefuelClient.QUERY_STEP_SIZE),
+        # construct parallel requests
+        request_params = copy.deepcopy(params)
+        request_params["offset"] = 0
+        request_params["max_events"] = 0
+
+        response = self._get(url=url, params=request_params).result()
+        if response.status_code != 200:
+            logger.error(
+                "Status code: {} received with response: {}",
+                response.status_code,
+                response.text,
             )
+        response_json = response.json()
+        return response_json.get("count", self.DEFAULT_MAX_QUERY_EVENTS)
 
-            response = self._get(url=url, params=params).result()
+    def _query_helper(
+        self, url: str, params: Dict = None, verbose: bool = False
+    ) -> pd.DataFrame:
+        self._event_count_helper(url, params)
+        model_name = params.get("model_name", None)
+        if not self._api_key:
+            logger.error(
+                f"API Key is absent or invalid: {self._api_key}. No event was logged"
+            )
+            return None
+        if model_name not in self._init_context.get("models", []):
+            logger.error(f"Model name: {model_name} does not exist for this team")
+            return None
 
+        max_events = min(
+            params.get("max_events"), self._event_count_helper(url, params)
+        )
+        offset = params.get("offset", 0)
+
+        # construct parallel requests
+        logger.info(f"Started fetching data. Will fetch {max_events} events ...")
+        futures = []
+        offset_starts = list(
+            range(offset, offset + max_events, RefuelClient.QUERY_STEP_SIZE)
+        )
+        for batch_num, offset_start in enumerate(offset_starts):
+            request_params = copy.deepcopy(params)
+            request_params["offset"] = offset_start
+            request_params["max_events"] = min(
+                max_events - offset_start, RefuelClient.QUERY_STEP_SIZE
+            )
+            future_obj = self._get(url=url, params=request_params)
+            future_obj.batch_num = batch_num
+            futures.append(future_obj)
+
+        # parse response from each request
+        batch_idx_to_events = {}
+        num_fetched = 0
+        for future in as_completed(futures):
+            response = future.result()
             if response.status_code != 200:
                 logger.error(
                     "Status code: {} received with response: {}",
                     response.status_code,
                     response.text,
                 )
-                return None
-
-            json_response = response.json()
-            result = json_response.get("result", [])
-            if len(result) == 0:
-                break
-
-            events += result
-
-            # check if API sends a `count` value and break when number of events exceeds the count
-            if json_response.get("count") and len(events) + offset >= json_response.get(
-                "count"
-            ):
-                break
+            else:
+                json_response = response.json()
+                result = json_response.get("result", [])
+                batch_idx_to_events[future.batch_num] = result
+                num_fetched += len(result)
+                if verbose:
+                    logger.info(f"Fetched {num_fetched} events so far.")
 
+        sorted_by_batch_idx = [item[1] for item in sorted(batch_idx_to_events.items())]
+        events = [item for sublist in sorted_by_batch_idx for item in sublist]
+        logger.info(f"Completed fetching data. {len(events)} events were fetched.")
         return pd.DataFrame.from_records(events)
 
     def query_events(
@@ -247,6 +291,9 @@ class RefuelClient:
         order_by: str = DEFAULT_ORDERING,
         offset: int = 0,
         max_events: int = DEFAULT_MAX_QUERY_EVENTS,
+        include_embeddings: bool = False,
+        embedding_column: str = None,
+        verbose: bool = False,
     ) -> pd.DataFrame:
         """
         Query events from the Refuel platform.
@@ -266,7 +313,7 @@ class RefuelClient:
                 f"Dataset name: {dataset_name} does not exist for model {model_name}"
             )
             return None
-        
+
         params = {
             "model_name": model_name,
             "dataset_name": dataset_name,
@@ -277,7 +324,13 @@ class RefuelClient:
             "max_events": max_events,
             "filters": json.dumps(filters),
         }
-        return self._query_helper(url=self._api_base_url + "/query", params=params)
+        if include_embeddings:
+            params["include_embeddings"] = True
+        if embedding_column:
+            params["embedding_column"] = embedding_column
+        return self._query_helper(
+            url=self._api_base_url + "/query", params=params, verbose=verbose
+        )
 
     def query_annotation_queue(
         self,
@@ -289,6 +342,7 @@ class RefuelClient:
         order_by: str = DEFAULT_ORDERING,
         offset: int = 0,
         max_events: int = DEFAULT_MAX_QUERY_EVENTS,
+        verbose: bool = False,
     ) -> pd.DataFrame:
         """
         Query events from the review queue for a Refuel model.
@@ -310,7 +364,63 @@ class RefuelClient:
             "filters": json.dumps(filters),
         }
         return self._query_helper(
-            url=self._api_base_url + "/query_annotation_queue", params=params
+            url=self._api_base_url + "/query_annotation_queue",
+            params=params,
+            verbose=verbose,
+        )
+
+    def clear_annotation_queue(
+        self,
+        model_name: str,
+        queue_name: str = DEFAULT_QUEUE_NAME,
+    ) -> pd.DataFrame:
+        """
+        Clear all events from a review queue for a Refuel model.
+
+        Args:
+            model_name (str): Refuel model to query
+            queue_name (str): Name of queue to clear
+
+        Returns:
+            pyfutures.Future:
+        """
+        params = {
+            "model_name": model_name,
+            "queue_name": queue_name,
+            "refuel_uuids": [],
+            "clear_queue": True,
+        }
+        return self._delete(
+            url=self._api_base_url + "/clear_annotation_queue",
+            params=params,
+        )
+
+    def delete_from_annotation_queue(
+        self,
+        model_name: str,
+        queue_name: str = DEFAULT_QUEUE_NAME,
+        refuel_uuids: list = [],
+    ) -> pd.DataFrame:
+        """
+        Delete events with specified refuel_uuids from a review queue for a Refuel model.
+
+        Args:
+            model_name (str): Refuel model to query
+            queue_name (str): Name of queue to clear
+            refuel_uuids (list): List of refuel_uuids to clear
+
+        Returns:
+            pyfutures.Future:
+        """
+        params = {
+            "model_name": model_name,
+            "queue_name": queue_name,
+            "refuel_uuids": refuel_uuids,
+            "clear_queue": False,
+        }
+        return self._delete(
+            url=self._api_base_url + "/clear_annotation_queue",
+            params=params,
         )
 
     def query_similar_events(
@@ -323,6 +433,7 @@ class RefuelClient:
         time_end: str = DATE_TOMORROW,
         filters: List[Dict] = [],
         max_events: int = DEFAULT_MAX_QUERY_EVENTS,
+        verbose: bool = False,
     ) -> pd.DataFrame:
         """
         Query events from the review queue for a Refuel model.
@@ -353,5 +464,7 @@ class RefuelClient:
             "fitlers": json.dumps(filters),
         }
         return self._query_helper(
-            url=self._api_base_url + "/query_similar_events", params=params
+            url=self._api_base_url + "/query_similar_events",
+            params=params,
+            verbose=verbose,
         )
