@@ -1,6 +1,6 @@
+import json
 from abc import ABC, abstractmethod
 from typing import Dict, List, Union
-import json
 
 import numpy as np
 from loguru import logger
@@ -19,10 +19,7 @@ class RefuelJSONEncoder(json.JSONEncoder):
 
 class BaseLoggingObj(ABC):
     def __init__(
-        self,
-        model_name: str,
-        dataset_name: str,
-        client_timestamp: str
+        self, model_name: str, dataset_name: str, client_timestamp: str
     ) -> None:
         self.model_name = model_name
         self.dataset_name = dataset_name
@@ -43,7 +40,7 @@ class Event(BaseLoggingObj):
         x: Dict,
         y_pred: Dict,
         y_true: Dict,
-        metadata: Dict
+        metadata: Dict,
     ) -> None:
         """
         Args:
@@ -63,28 +60,25 @@ class Event(BaseLoggingObj):
     def serialize(self) -> Dict:
         if not self.model_name:
             logger.error(
-                f"No model name specified for logging event: {self.model_name}")
+                f"No model name specified for logging event: {self.model_name}"
+            )
             return {}
 
         data = {
-            'model_name': self.model_name,
-            'dataset_name': self.dataset_name,
-            'x': self.x,
-            'y_pred': self.y_pred,
-            'y_true': self.y_true,
-            'metadata': self.metadata,
-            'refuel_client_timestamp': self.client_timestamp
+            "model_name": self.model_name,
+            "dataset_name": self.dataset_name,
+            "x": self.x,
+            "y_pred": self.y_pred,
+            "y_true": self.y_true,
+            "metadata": self.metadata,
+            "refuel_client_timestamp": self.client_timestamp,
         }
         return json.dumps(data, cls=RefuelJSONEncoder)
 
 
 class EventBatch(BaseLoggingObj):
     def __init__(
-        self,
-        model_name: str,
-        dataset_name: str,
-        client_timestamp: str,
-        events: List
+        self, model_name: str, dataset_name: str, client_timestamp: str, events: List
     ) -> None:
         """_summary_
 
@@ -99,20 +93,21 @@ class EventBatch(BaseLoggingObj):
     def serialize(self) -> List:
         if not self.model_name:
             logger.error(
-                f"No model name specified for logging event: {self.model_name}")
+                f"No model name specified for logging event: {self.model_name}"
+            )
             return []
 
         data = []
         for event in self.events:
             data.append(
                 {
-                    'model_name': self.model_name,
-                    'dataset_name': self.dataset_name,
-                    'x': event.get('x'),
-                    'y_pred': event.get('y_pred'),
-                    'y_true': event.get('y_true'),
-                    'metadata': event.get('metadata'),
-                    'refuel_client_timestamp': self.client_timestamp
+                    "model_name": self.model_name,
+                    "dataset_name": self.dataset_name,
+                    "x": event.get("x"),
+                    "y_pred": event.get("y_pred"),
+                    "y_true": event.get("y_true"),
+                    "metadata": event.get("metadata"),
+                    "refuel_client_timestamp": self.client_timestamp,
                 }
             )
         return json.dumps(data, cls=RefuelJSONEncoder)
@@ -124,8 +119,8 @@ class DatasetEvent(BaseLoggingObj):
         model_name: str,
         dataset_name: str,
         client_timestamp: str,
-        s3_path: str,
-        dataset_config: dict
+        cloud_uri: str,
+        dataset_config: dict,
     ) -> None:
         """_summary_
 
@@ -133,26 +128,27 @@ class DatasetEvent(BaseLoggingObj):
             model_name (str): model dataset belongs to
             dataset_name (str): name of the dataset
             client_timestamp (str): provided timestamp
-            s3_path (str): path to dataset in s3
+            cloud_uri (str): cloud uri or pre-signed uri to dataset in cloud storage
         """
         super().__init__(model_name, dataset_name, client_timestamp)
-        self.s3_path = s3_path
+        self.cloud_uri = cloud_uri
         self.dataset_config = dataset_config
-        self.dataset_config.setdefault('type', 'CSV')
-        self.dataset_config.setdefault('delimiter', ',')
+        self.dataset_config.setdefault("type", "CSV")
+        self.dataset_config.setdefault("delimiter", ",")
 
     def serialize(self) -> List:
         if not self.model_name:
             logger.error(
-                f"No model name specified for logging event: {self.model_name}")
+                f"No model name specified for logging event: {self.model_name}"
+            )
             return {}
 
         data = {
-            'model_name': self.model_name,
-            'dataset_name': self.dataset_name,
-            's3_path': self.s3_path,
-            'dataset_config': json.dumps(self.dataset_config),
-            'refuel_client_timestamp': self.client_timestamp
+            "model_name": self.model_name,
+            "dataset_name": self.dataset_name,
+            "cloud_uri": self.cloud_uri,
+            "dataset_config": json.dumps(self.dataset_config),
+            "refuel_client_timestamp": self.client_timestamp,
         }
 
         return json.dumps(data)
